/*
 * This is a pre-process program to get the index file
 * It is a java class to get normInfl|Ciataions from ./data/inflVars.data:
 * infl|POS|inflections|Eui|base form|citation form
 *
 */

import java.util.*;
import java.io.*;

import gov.nih.nlm.nls.lvg.Lib.*;
import gov.nih.nlm.nls.lvg.Flows.*;
import gov.nih.nlm.nls.lvg.Api.*;

public class GenIndexFile
{
	public static void main(String[] args)
	{
		// TBU: To be updated 
		String dataDir = 
			"/nfsvol/crfiler-lex/Lu/Development/LVG/Components/Applications/MetaMapNorm/data/";

		if(args.length == 1)
		{
			dataDir = args[0];
		}
		else if(args.length > 0)
		{
			System.err.println("Usage: java PreMetaMapNorm <dataDir>");
			System.exit(0);
		}

		String inFile = dataDir + "inflVars.data";
		String citationsFile = dataDir +  "normCitations.data";
		String inflVarsFile = dataDir +  "normInflVars.data";

		// clean the file
		GenIndexFile.GetNormCitations(inFile, citationsFile);
		GenIndexFile.GetNormInflVars(inFile, inflVarsFile);
	}

	public static void GetNormInflVars(String inFile, String outFile)
	{
		// TBU: To be updated 
		String lvgConfigFile = 
			"/usr/local/Deploy/Projects/LVG/lvg2010/data/config/lvg.properties";
		LvgApi lvgApi = new LvgApi(lvgConfigFile);

		// key: MetaNorm(inflVars): No Punc + lowercase of inflVars
		// value: all possible orginal inflections
		Hashtable<String, HashSet<String>> normInflVars 
			= new Hashtable<String, HashSet<String>>();

		// Read in inFile 1
		int lineNo = 0;
		String line = null;

		// must use try/catch clause with class FileReader since it throws
		try
		{
			BufferedReader in = new BufferedReader(new FileReader(inFile));

			// read in file: line by line
			while((line = in.readLine()) != null)
			{
				lineNo++;
				// skip the line if it is empty or comments (#)
				if((line.length() > 0) && (line.charAt(0) != '#'))
				{
					// get fields: use '|' as delimiter to parse token
					StringTokenizer buf = new StringTokenizer(line, "|");
					String inflVar = buf.nextToken();	// 1st field
					String pos = buf.nextToken();	// 2nd field
					String infl = buf.nextToken();	// 3rd field
					String eui = buf.nextToken();	// 4th field
					String base = buf.nextToken();	// 5th field
					String citation = buf.nextToken();	// 6th field

					// normalize the inflVar
					String normInflVar = MetaMapNorm.Mutate(inflVar, lvgApi);

					// add to normInflVars
					if(normInflVars.containsKey(normInflVar) == false)
					{
						HashSet<String> inflVars = new HashSet<String>();
						inflVars.add(inflVar);
						normInflVars.put(normInflVar, inflVars); 
					}
					else	// has the normInflVars
					{
						HashSet<String> inflVars 
							= normInflVars.get(normInflVar);
						inflVars.add(inflVar);
						normInflVars.put(normInflVar, inflVars);//set keyValue
					}
				}

				if((lineNo%100000) == 0)
				{
					System.out.println("-- line no: " + lineNo);
				}
			}
			System.out.println("-- Total line no: " + lineNo);

			// clean up
			in.close();
			lvgApi.CleanUp();

			// print out normInflVars
			System.out.println("-- Total key (normINflVars): " 
				+ normInflVars.size());

			BufferedWriter out = new BufferedWriter(new FileWriter(outFile));
			int keyNo = 0;
			Enumeration<String> keys = normInflVars.keys();
			while(keys.hasMoreElements() == true)
			{
				String normInflVar = keys.nextElement();
				HashSet<String> inflVars = normInflVars.get(normInflVar);
				keyNo++;
				String outStr = normInflVar;

				Iterator<String> it = inflVars.iterator();
				while(it.hasNext() == true)
				{
					String inflVar = it.next();
					outStr += "|" + inflVar;
				}
				out.write(outStr);
				out.newLine();

				if((keyNo%100000) == 0)
				{
					System.out.println("-- key no: " + keyNo);
				}
			}

			System.out.println("-- Total key no: " + keyNo);

			// close inFile and outFile
			out.close();
		}
		catch(Exception e)
		{
			System.out.println("-------------Exception: " + e.toString());
		}
	}

	// this method is not needed due to requirement change
	public static void GetNormCitations(String inFile, String outFile)
	{
		// TBU: To be updated 
		String lvgConfigFile = 
			"/usr/local/Deploy/Projects/LVG/lvg2010/data/config/lvg.properties";
		LvgApi lvgApi = new LvgApi(lvgConfigFile);

		// key: MetaNorm(infls): No Punc + lowercase of inflVars
		// value: citation form
		Hashtable<String, HashSet<String>> normCitations 
			= new Hashtable<String, HashSet<String>>();

		// Read in inFile 1
		int lineNo = 0;
		String line = null;

		// must use try/catch clause with class FileReader since it throws
		try
		{
			BufferedReader in = new BufferedReader(new FileReader(inFile));

			// read in file: line by line
			while((line = in.readLine()) != null)
			{
				lineNo++;
				// skip the line if it is empty or comments (#)
				if((line.length() > 0) && (line.charAt(0) != '#'))
				{
					// get fields: use '|' as delimiter to parse token
					StringTokenizer buf = new StringTokenizer(line, "|");
					String inflVar = buf.nextToken();	// 1st field
					String pos = buf.nextToken();	// 2nd field
					String infl = buf.nextToken();	// 3rd field
					String eui = buf.nextToken();	// 4th field
					String base = buf.nextToken();	// 5th field
					String citation = buf.nextToken();	// 6th field

					// normalize the inflVar
					String normInflVar = MetaMapNorm.Mutate(inflVar, lvgApi);

					// add to normCitations
					if(normCitations.containsKey(normInflVar) == false)
					{
						HashSet<String> citations = new HashSet<String>();
						citations.add(citation);
						normCitations.put(normInflVar, citations); 
					}
					else	// has the normInflVar
					{
						HashSet<String> citations 
							= normCitations.get(normInflVar);
						citations.add(citation);
						normCitations.put(normInflVar, citations);//set keyValue
					}
				}

				if((lineNo%100000) == 0)
				{
					System.out.println("-- line no: " + lineNo);
				}
			}
			System.out.println("-- Total line no: " + lineNo);

			// clean up
			in.close();
			lvgApi.CleanUp();

			// print out normCitations
			System.out.println("-- Total key (normCitations): " 
				+ normCitations.size());

			BufferedWriter out = new BufferedWriter(new FileWriter(outFile));
			int keyNo = 0;
			Enumeration<String> keys = normCitations.keys();
			while(keys.hasMoreElements() == true)
			{
				String normInflVar = keys.nextElement();
				HashSet<String> citations = normCitations.get(normInflVar);
				keyNo++;
				String outStr = normInflVar;

				Iterator<String> it = citations.iterator();
				while(it.hasNext() == true)
				{
					String citation = it.next();
					outStr += "|" + citation;
				}
				out.write(outStr);
				out.newLine();

				if((keyNo%100000) == 0)
				{
					System.out.println("-- key no: " + keyNo);
				}
			}

			System.out.println("-- Total key no: " + keyNo);

			// close inFile and outFile
			out.close();
		}
		catch(Exception e)
		{
			System.out.println("-------------Exception: " + e.toString());
		}
	}

	// data member
}
