/*
 * This program is a java class to get citation form from input by utilizing
 * MetaMapNorm, -f:g:rs:o:l
 *
 */

import java.util.*;
import java.io.*;

import gov.nih.nlm.nls.lvg.Lib.*;
import gov.nih.nlm.nls.lvg.Flows.*;
import gov.nih.nlm.nls.lvg.Api.*;

public class GetLexFormInput
{
	public static void main(String[] args)
	{
		if(args.length > 0)
		{
			System.err.println("Usage: java GetLexFormInput");
			System.exit(0);
		}

		// init
		//String inStr = "4-colour";
		//String inStr = "University of california Los Angeles";
		//String inStr = "HACKERS";
		String inStr = "2 compartment models";
		String lvgConfigFile = 
			"/usr/local/Deploy/Projects/LVG/lvg2010/data/config/lvg.properties";
		LvgApi lvgApi = new LvgApi(lvgConfigFile);

		/**
		// TBU: to be updated
		String inFile = 
			"/nfsvol/crfiler-lex/Lu/Development/LVG/Components/Applications/MetaMapNorm/data/normCitations.data";

		// get citation from input: 
		Hashtable<String, Vector<String>> normCitations
			= GetLexFormInput.ReadMetaNormsFromFile(inFile, false);

		// get citations
		Vector<String> outStrs = GetLexFormInput.Mutate(inStr, lvgApi, 
			normCitations);
		**/

		// TBU: to be updated
		String inFile = 
			"/nfsvol/crfiler-lex/Lu/Development/LVG/Components/Applications/MetaMapNorm/data/normInflVars.data";

		// get inflVars from input (Lexical Lookup): 
		Hashtable<String, Vector<String>> normInflVars
			= GetLexFormInput.ReadMetaNormsFromFile(inFile, false);

		// get citations
		Vector<String> outStrs = GetLexFormInput.Mutate(inStr, lvgApi, 
			normInflVars);

		lvgApi.CleanUp();

		// print out citations
		System.out.println("-- input: [" + inStr + "]");
		System.out.print("-- Output: [");
		for(int i = 0; i < outStrs.size(); i++)
		{
			if(i == outStrs.size()-1)
			{
				System.out.print(outStrs.elementAt(i));
			}
			else
			{
				System.out.print(outStrs.elementAt(i) + ", ");
			}
		}
		System.out.println("]");
	}

	public static Hashtable<String, Vector<String>> ReadMetaNormsFromFile
		(String inFile, boolean verbose)
	{
		Hashtable<String, Vector<String>> metaNorms = 
			new Hashtable<String, Vector<String>>();

		// Read in inFile 1
		int lineNo = 0;
		String line = null;

		// must use try/catch clause with class FileReader since it throws
		try
		{
			BufferedReader in = new BufferedReader(new FileReader(inFile));

			// read in file: line by line
			while((line = in.readLine()) != null)
			{
				lineNo++;
				// skip the line if it is empty or comments (#)
				if((line.length() > 0) && (line.charAt(0) != '#'))
				{
					// get normInfl and citations: use '|' as delimiter 
					StringTokenizer buf = new StringTokenizer(line, "|");
					String normInflVar = buf.nextToken();	// 1st field

					// could be inflVars or citations
					Vector<String> values = new Vector<String>();
					while(buf.hasMoreTokens() == true)
					{
						String value = buf.nextToken();	// next field
						values.addElement(value);
					}

					// put to metaNorms
					metaNorms.put(normInflVar, values);
				}

				if(verbose && (lineNo%100000) == 0)
				{
					System.out.println("-- line no: " + lineNo);
				}
			}
			// clean up
			in.close();

			if(verbose)
			{
				System.out.println("-- Total line no: " + lineNo);
			}
		}
		catch(Exception e)
		{
			System.out.println("** ERR@Exception: " + e.toString());
		}

		return metaNorms;
	}

	public static Vector<String> Mutate(String inStr, LvgApi lvgApi,
		Hashtable<String, Vector<String>> metaNorms)
	{
		// 1. normalized input str
		String normStr = MetaMapNorm.Mutate(inStr, lvgApi);

		// 2. find values (inflVars or citations) from hash table
		Vector<String> values = metaNorms.get(normStr);

		if(values == null)
		{
			values = new Vector<String>();
		}

		return values;
	}

	// data member
}
