/*
 * This program is a java class to test get citation form from input by 
 * utilizing MetaMapNorm, -f:g:rs:o:l and compare to RESULTS (from 'C-code')
 *
 */

import java.util.*;
import java.io.*;

import gov.nih.nlm.nls.lvg.Lib.*;
import gov.nih.nlm.nls.lvg.Flows.*;
import gov.nih.nlm.nls.lvg.Api.*;

public class GetLexFormInputTest
{
	public static void main(String[] args)
	{
		// TBU: update top directory
		String topDir = 
			"/nfsvol/crfiler-lex/Lu/Development/LVG/Components/Applications/MetaMapNorm/";
		String normInflVarsFile = topDir + "/data/normInflVars.data";
		String inFile = topDir + "/data/RESULTS";
		String outFile = topDir + "/data/RESULT.out";
		String diffFile = topDir + "/data/RESULT.diff";

		if(args.length > 0)
		{
			System.err.println("Usage: java GetLexFormInputTest");
			System.exit(0);
		}

		// init
		// TBU: update lvg config
		String lvgConfigFile = 
			"/usr/local/Deploy/Projects/LVG/lvg2010/data/config/lvg.properties";
		LvgApi lvgApi = new LvgApi(lvgConfigFile);

		// test citations
		Hashtable<String, Vector<String>> normInflVars
			= GetLexFormInput.ReadMetaNormsFromFile(normInflVarsFile, true);

		// get citations
		TestFile(inFile, outFile, diffFile, lvgApi, normInflVars);
		lvgApi.CleanUp();
	}

	private static void TestFile(String inFile, String outFile, String diffFile,
		LvgApi lvgApi, Hashtable<String, Vector<String>> metaNorms)
	{
		String line = null;
		int lineNo = 0;
		int diffNo = 0;

		// must use try/catch clause with class FileReader since it throws
		try
		{
			BufferedReader in = new BufferedReader(new FileReader(inFile));
			BufferedWriter out = new BufferedWriter(new FileWriter(outFile));
			BufferedWriter diff = new BufferedWriter(new FileWriter(diffFile));

			// read in file: line by line
			while((line = in.readLine()) != null)
			{
				lineNo++;
				// skip the line if it is empty or comments (#)
				if((line.length() > 0) && (line.charAt(0) != '#'))
				{
					// use '|' as delimiter to parse toke
					StringTokenizer buf = new StringTokenizer(line, "|");
					String inStr = buf.nextToken();   // 1st field
					String inflStr = buf.nextToken();   // 2nd field
					String cStr = buf.nextToken();   // 3rd field

					Vector<String> outStrs = GetLexFormInput.Mutate(inStr,
						lvgApi, metaNorms);

					String outStr = ReFormat(outStrs);

					out.write(inStr + "|" + outStr);
					out.newLine();

					// compare results to inflectional string
					if(outStrs.contains(inflStr) == false)
					{
						diffNo++;
						diff.write(lineNo + "|" + inStr + "|" + inflStr + "|" 
							+ outStr);
						diff.newLine();
					}
				}

				if((lineNo%100000) == 0)
				{
					System.out.println("-- line no: " + lineNo);
				}
			}
			System.out.println("-- Total line no: " + lineNo);
			System.out.println("-- Total diff no: " + diffNo);

			// clean up
			in.close();
			out.close();
			diff.close();
		}
		catch(Exception e)
		{
			System.out.println("** ERR@Exception: " + e.toString());
		}
	}

	// change format to: [r1, r2, ...]
	private static String ReFormat(Vector<String> inStrs)
	{
		String outStr = "[";
		
		int size = inStrs.size();
		for(int i = 0; i < size; i++)
		{
			if(i == size-1)
			{
				outStr += inStrs.elementAt(i);
			}
			else
			{
				outStr += inStrs.elementAt(i) + ",";
			}
		}
		outStr += "]";

		return outStr;
	}

	// data member
}
